PRESERVE.
SET PRINTBACK=NONE ERRORS=NONE.
DO IF ($casenum=1).
  PRINT /'Macro !OPdummy V2009.04.29 loaded.'
        /'Execute !OPdummy HELP. for printing Help & Macro bibliographic reference.'/.
END IF.
EXECUTE.
DEFINE !OPdummy ( X = !CHAREND('/') 
                 /K = !CHAREND('/')
            /PREFIX = !CHAREND('/')
               /ADD = !CHAREND('/')!DEFAULT(Y)
               /NST = !CHAREND('/')
              /HELP = !CHAREND('/')!DEFAULT(1) ).
PRESERVE.
SET PRINTBACK=NONE LENGHT=NONE ERRORS=NONE WIDTH=132 .
!LET !VERSION=!QUOTE('2009.04.29').
!LET !YEAR=!QUOTE(!SUBSTR(!VERSION,2,4)).
DO IF ($casenum=1).
   PRINT //'Macro !OPdummy V' !VERSION ' (c)JM.Domenech & JB.Navarro'.
END IF.
!IF (!HELP<>1) !THEN.
*  Print the help.
   DO IF ($casenum=1).
      PRINT /'******************************************************************************'
            /'* ORTHOGONAL POLYNOMIALS GENERATION                                          *'
            /'* Created 2007.10.24 Last revised ' !VERSION                                '*'78
            /'* (c) JM.Domenech & JB.Navarro                                               *'
            /'* Programmer: R.Sesma                                                        *'
            /'*                                                                            *'
            /'* Vancouver reference:                                                       *'
            /'* Domenech JM, Navarro JB. Macro !OPdummy for SPSS Statistics.               *'
            /'*    Orthogonal Polynomials Generation [computer program].                   *'
            /'*    V' !VERSION '. Bellaterra: Universitat Autnoma de Barcelona; ' !YEAR '.' '*'78
            /'*    Available from: http://www.metodo.uab.cat/macros.htm.                   *'
            /'*                                                                            *'
            /'* Macro Call (tested for SPSS 15.0 and higher):                              *'
            /'* !OPdummy X = Name of categorical variable containing the metric            *'
            /'*          K = Number of categories                    (Integer >=3 & <=20)  *'
            /'*    [PREFIX]= Prefix of dummy variable names          (<= 8 characters)     *'
            /'*              (Suffix is a degree of orthogonal polynomial: 01, 02, ...)    *'
            /'*              by default use X categorical variable name as a prefix        *'
            /'*       [ADD]= Add Dummy variables in the Work Data File (Y: Yes;  N: None)  *'
            /'*                                                         by default: Y      *'
            /'*      [/NST]= Name of the study (Label)                                     *'
            /'*      [/HELP] Print Macro call arguments and Samples of calls               *'
            /'*                                                                            *'
            /'* Note: Parameters enclosed in square brackets [ ] are optional              *'
            /'*                                                                            *'
            /'* WARNING: The macro provides wrong results if your data are filtered.       *'
            /'*          To work with a set of cases you must use SELECT IF procedure in   *'
            /'*          order to create a new data file containing only selected cases.   *'
            /'*                                                                            *'
            /'* Samples of calls:                                                          *'
            /'* !OPdummy HELP.                                                             *'
            /'* !OPdummy NST= "Name of the Study" /X=Treatment /K=3 /PREFIX=Treat .        *'
            /'* !OPdummy X=Dose /K=4 /ADD=N.                                               *'
            /'******************************************************************************'.
   END IF.
   EXECUTE.
!ELSE. 
*  Test parameters.
   !IF ((!X=!NULL !OR !K=!NULL) !OR (!ADD<>Y !AND !ADD<>N)) !THEN.
      DO IF ($casenum=1).
         !IF ((!X=!NULL !OR !K=!NULL)) !THEN.
            PRINT /'ERROR: Missing parameter'.
         !ELSE.
            PRINT /'ERROR: Parameter(s) with wrong values'.
         !IFEND.
      END IF.
   !ELSE.
*     Sort by !X, use @$CASENUM to restore previous order.
      COMPUTE @$CASENUM=$CASENUM.
      SORT CASES BY !X (A).

*     Var @temp will be useful for printing and compute the number of categories of !X.
      COMPUTE @temp=(LAG(!X)<>!X).
      IF (NOT MISSING(!X) AND MISSING(@temp)) @temp=1.
      EXECUTE.

      !LET !X0=!CONCAT(!X,'0').        /*Compute degree 0 polynomial.
      COMPUTE !X0=1.
      EXECUTE.

*     Compute @DEGREE with the number of categories of !X.
      AGGREGATE
        /OUTFILE=*
        MODE=ADDVARIABLES
        /BREAK=!X0
        /@DEGREE = SUM(@temp).
      FORMATS @DEGREE (F2).

      !LET !DEGREE=''.     /*Degree = K-1.
      !LET !DEGREE11='0'. 
      !LET !DEGREE21='0'. 
      !DO !I = 1 !TO !K.
         !IF (!I<>!K) !THEN.
            !LET !DEGREE=!CONCAT(!DEGREE,' ').
            !IF (!LENGTH(!DEGREE)=11) !THEN.
               !LET !DEGREE11='1'.
            !IFEND.
            !IF (!LENGTH(!DEGREE)=21) !THEN.
               !LET !DEGREE21='1'.
            !IFEND.
         !IFEND.
      !DOEND.
      !LET !DEGREE=!LENGTH(!DEGREE).
      !LET !NDEGREE=!NULL.
      !IF (!DEGREE21='0') !THEN.
         !LET !NDEGREE=!DEGREE.
      !ELSE.
         !LET !NDEGREE=20.
      !IFEND.

      !LET !SUFFIX='010203040506070809101112131415161718192021222324252627282930313234353637383940'.

*     Polynomials generated with Christoffel-Darboux' recurrence relation.
*     Loop to compute the orthogonal polynomials.
      !LET !Xminus1=!X0.
      !LET !VARNAMES=''.
      !LET !COUNT=' '.
      !DO !I = 1 !TO !DEGREE.
         !LET !SUFIXi=!SUBSTRING(!SUFFIX,!LENGTH(!COUNT),2).      /*Build the name of the dummy variable.
         !IF (!PREFIX=!NULL) !THEN.
            !LET !Xi=!CONCAT(!X,!SUFIXi).
         !ELSE.
            !LET !Xi=!CONCAT(!PREFIX,!SUFIXi).           /*Use PREFIX for the name.
         !IFEND.
         
         !LET !Ai=!CONCAT('A',!I).
         !LET !Bi=!CONCAT('B',!I).
         !LET !Ci=!CONCAT('C',!I).
         
         COMPUTE T=!X*(!Xminus1**2).
         AGGREGATE
           /OUTFILE=* MODE=ADDVARIABLES
           /BREAK=!X0
           /!Bi = MEAN(T).
   
         COMPUTE T=!X*T.
         AGGREGATE
           /OUTFILE=* MODE=ADDVARIABLES
           /BREAK=!X0
           /!Ai = MEAN(T).
   
         !IF (!I=1) !THEN.
            COMPUTE !Ci=0.
            !LET !Xminus2=!X0.
         !ELSE.
            COMPUTE T=!X*!Xminus1*!Xminus2.
            AGGREGATE
              /OUTFILE=* MODE=ADDVARIABLES
              /BREAK=!X0
              /!Ci = MEAN(T).
         !IFEND.
   
         COMPUTE !Ai=1/SQRT(!Ai - !Bi**2 - !Ci**2).
         COMPUTE !Bi=-!Ai*!Bi.
         COMPUTE !Ci= !Ai*!Ci.
         COMPUTE !Xi=(!Ai*!X+!Bi)*!Xminus1-!Ci*!Xminus2.
         EXECUTE.
         
         !LET !Xminus2=!Xminus1.         
         !LET !Xminus1=!Xi.
         
         !LET !VARNAMES=!CONCAT(!VARNAMES,' ',!Xi).
         !LET !COUNT=!CONCAT(!COUNT,'  ').
         
         DELETE VARIABLES !Ai !Bi !Ci.
         
         FORMATS !Xi (F10.7).
      !DOEND.
            
      STRING @XNAME (A8).           /*Name of the categorical variable, truncated if >8 chars.
      COMPUTE @XNAME=!QUOTE(!X).
      IF (!LENGTH(!X)>8) @XNAME=CONCAT(SUBSTR(@XNAME,1,7),'.').
      IF (!LENGTH(!X)<8) @XNAME=LPAD(RTRIM(@XNAME),8).

      !IF (!PREFIX=!NULL) !THEN.
         !LET !NAME=!QUOTE(!X).
      !ELSE.
         !LET !NAME=!QUOTE(!PREFIX).
      !IFEND.
      
      !IF (!DEGREE11='1') !THEN.    /*If K-1>10, build the params for printing in 2 rows.
         !LET !VARNAMES1=''.
         !LET !VARNAMES2=''.
         !LET !COUNT=' '.
         !LET !COUNT2=''.
         !LET !FLAG='0'.
         !DO !VAR !IN (!VARNAMES).
            !IF (!LENGTH(!COUNT)=21) !THEN.
               !BREAK.
            !IFEND.
            !IF (!LENGTH(!COUNT)=11) !THEN.
               !LET !FLAG='1'.
            !IFEND.
            !IF (!FLAG='0') !THEN.
               !LET !VARNAMES1=!CONCAT(!VARNAMES1,' ',!VAR).
            !ELSE.
               !LET !VARNAMES2=!CONCAT(!VARNAMES2,' ',!VAR).
               !LET !COUNT2=!CONCAT(!COUNT2,' ').
            !IFEND. 
            !LET !COUNT=!CONCAT(!COUNT,' ').
         !DOEND.
         !LET !COUNT2=!LENGTH(!COUNT2).
      !IFEND.

      DO IF ($CASENUM=1).     /*Print header and param NST.
         PRINT /'ORTHOGONAL POLYNOMIALS GENERATION'.         
         !IF (!NST !NE !NULL) !THEN .	/*Print NST if available.
            STRING #NST (A131).
            COMPUTE #NST = CONCAT('STUDY: ',!NST).
            PRINT /#NST.
         !IFEND.
      END IF.

      COMPUTE #COUNT=1.       /*Build the #NAME1 to !NAMEn names of the K-1 orthogonal polynomials, truncated if >8 chars.
      COMPUTE #TRUNC=0.
      IF (!LENGTH(!NAME)>8) #TRUNC=1.
      !LET !NAMEn = !CONCAT('NAME',!NDEGREE).
      STRING NAME1 to !NAMEn (A11).
      DO REPEAT NAME=NAME1 to !NAMEn.
         DO IF (#TRUNC=0).
            DO IF (#COUNT<10).
               COMPUTE NAME = LPAD(RTRIM(CONCAT(!NAME,'0',STRING(#COUNT,F1))),10).
            ELSE.
               COMPUTE NAME = LPAD(RTRIM(CONCAT(!NAME,STRING(#COUNT,F2))),10).
            END IF.
         ELSE.
            DO IF (#COUNT<10).
               COMPUTE NAME = CONCAT(SUBSTR(!NAME,1,7),'.0',STRING(#COUNT,F1)).
            ELSE.
               COMPUTE NAME = CONCAT(SUBSTR(!NAME,1,7),'.',STRING(#COUNT,F2)).
            END IF.
         END IF.
         COMPUTE #COUNT=#COUNT+1.
      END REPEAT.
      EXECUTE.
      
      !IF (!DEGREE11='1') !THEN.
*        If K-1>10 print the orthogonal coefficients using 2 rows.
         DO IF ($CASENUM=1).
            PRINT//@XNAME(A8) '  Orthogonal Polynomial Coefficients'.
            PRINT /'(metric)  '  NAME1 to NAME10 (10(A11)).
         END IF.
         TEMPORARY.
         SELECT IF (@temp=1).
         PRINT /!X(F8.3) '  ' !VARNAMES1.
         EXECUTE.

         DO IF ($CASENUM=1).
            PRINT//@XNAME(A8) '  Orthogonal Polynomial Coefficients'.
            PRINT /'(metric)  '  NAME11 to !NAMEn (!COUNT2(A11)).
         END IF.
         TEMPORARY.
         SELECT IF (@temp=1).
         PRINT /!X(F8.3) '  ' !VARNAMES2.
         EXECUTE.
      !ELSE.   
*        Print orthogonal coefficients.
         DO IF ($CASENUM=1).
            PRINT//@XNAME(A8) '  Orthogonal Polynomial Coefficients'.
            PRINT /'(metric)  '  NAME1 to !NAMEn (!DEGREE(A11)).
         END IF.
         TEMPORARY.
         SELECT IF (@temp=1).
         PRINT /!X(F8.3) '  ' !VARNAMES.
         EXECUTE.
      !IFEND.

      !IF (!ADD=Y) !THEN.
         DO IF ($CASENUM=1).
            PRINT //'NOTE: ' !QUOTE(!DEGREE) ' Dummy variables were added to the Working Data File'.
         END IF.
         EXECUTE.         
      !ELSE.
*        If ADD=N, print syntax for generate the dummy variables and erase created vars.
         DO IF ($CASENUM=1).
            PRINT //'*PROCEDURES FOR GENERATE DUMMY VARIABLES.'.
         END IF.
         EXECUTE.

         !DO !VAR !IN (!VARNAMES).
            DO IF ($CASENUM=1).
               PRINT /'COMPUTE ' !QUOTE(!VAR) '= $SYSMIS.'.
            END IF.
            EXECUTE.
         !DOEND.

         TEMPORARY.
         SELECT IF (@temp=1).         
         PRINT /'DO IF (' !QUOTE(!X) '=' !X(F8.3) ').'.
         !DO !VAR !IN (!VARNAMES).
            PRINT /'   COMPUTE ' !QUOTE(!VAR) '=' !VAR(F15.12) '.'.
         !DOEND.
         PRINT /'END IF.'.
         EXECUTE.

         DO IF ($CASENUM=1).
            PRINT /'EXECUTE.'.
         END IF.
         EXECUTE.
      !IFEND.

      COMPUTE #K=!K.          /*Error message for K<DEGREE.
      COMPUTE #DEGREE=@DEGREE.
      DO IF (#K<#DEGREE).
         DO IF ($CASENUM=1).
            PRINT//'CATASTROPHIC ERROR!!!'.
            PRINT /'Erroneous dummy variables decomposition because K parameter'.
            PRINT /'is less than the number of categories of independent variable X.'.
            PRINT /'Parameter must be K =' @DEGREE.
         END IF.
      END IF.

      !IF (!DEGREE21='1') !THEN.        /*Warning for more than 20 vars generated.
         DO IF ($CASENUM=1).
            PRINT//'WARNING: More than 20 dummy variables were generated.'.
            PRINT /'         Only the first 20 orthogonal polynomial coefficients are printed'.
         END IF.
      !IFEND.

      SORT CASES BY @$CASENUM (A).        /*Restore previous order.

      !IF (!ADD=Y) !THEN.                 /*Erase temporal vars.
         DELETE VARIABLES !X0 T @temp @$CASENUM NAME1 to !NAMEn @DEGREE @XNAME.
      !ELSE.
         DELETE VARIABLES !X0 @$CASENUM T @temp NAME1 to !NAMEn @DEGREE @XNAME !VARNAMES.
      !IFEND.
   !IFEND.
!IFEND.
EXECUTE.
RESTORE.
!ENDDEFINE.
RESTORE.

