PRESERVE.
SET PRINTBACK=NONE ERRORS=NONE.
DO IF ($casenum=1).
  PRINT /'Macro !BINOM V2010.11.22 loaded.'
        /'Execute !BINOM HELP. for printing Help & Macro bibliographic reference.'/.
END IF.
EXECUTE.
DEFINE !BINOM ( K =!CHAREND('/') 
               /N =!CHAREND('/')
               /P =!CHAREND('/') 
             /NST =!CHAREND('/')
            /HELP =!CHAREND('/')!DEFAULT(1) ).
PRESERVE.
SET PRINTBACK=NONE ERRORS=NONE LENGHT=NONE WIDTH=132 .
!LET !VERSION=!QUOTE('2010.11.22').
!LET !YEAR=!QUOTE(!SUBSTR(!VERSION,2,4)).
DO IF ($casenum=1).
   PRINT //'Macro !BINOM V' !VERSION ' (c)JM.Domenech & R.Granero'.
END IF.
TEMPORARY.
N 1.
!IF (!HELP<>1) !THEN.
*  Print HELP.
   DO IF ($casenum=1).
      PRINT /'******************************************************************************'
            /'* BINOMIAL DISTRIBUTION                                                      *'
            /'* Created 1999.12.13 Last revised ' !VERSION                                '*'78
            /'* (c) JM.Domenech & R.Granero                                                *'
            /'* Programmers: A.Bonillo & R.Sesma                                           *'
            /'*                                                                            *'
            /'* Vancouver reference:                                                       *'
            /'* Domenech JM, Granero R. Macro !BINOM for SPSS Statistics.                  *'
            /'*    Binomial Distribution [computer program].                               *'
            /'*    V' !VERSION '. Bellaterra: Universitat Autnoma de Barcelona; ' !YEAR '.' '*'78
            /'*    Available from: http://www.metodo.uab.cat/macros.htm.                   *'
            /'*                                                                            *'
            /'* Macro Call (tested for SPSS 15.0 and higher):                              *'
            /'* !BINOM  P= Probability of the event                     (0 < P < 1)        *'
            /'*        /N= Sample size                                  (Integer > 0)      *'
            /'*      [/K]= List with the number of events in the sample (Integer >= 0)     *'
            /'*                                     by default 0 THRU N                    *'
            /'*    [/NST]= Name of the study (Label)                                       *'
            /'*    [/HELP] Print Macro call arguments and Samples of calls                 *'
            /'*                                                                            *'
            /'* Note: Parameters enclosed in square brackets [ ] are optional              *'
            /'*                                                                            *'
            /'* Samples of calls:                                                          *'
            /'*  !BINOM HELP.                                                              *'
            /'*  !BINOM P=0.50 /N=18.                                                      *'
            /'*  !BINOM K=1,2,50,74 /P=0.2 /N=76.                                          *'
            /'*  !BINOM K=0 THRU 6 /P=0.15 /N=100 /NST="Name of the Study".                *'
            /'******************************************************************************'.
   END IF.
   EXECUTE.
!ELSE.
*  Testing the parameters.
   COMPUTE #ERROR=0.
   !IF (!P=!NULL !or !N=!NULL) !THEN.
      PRINT /'ERROR: Missing parameter'.
   !ELSE.      
      COMPUTE #P=$SYSMIS.
      COMPUTE #N=$SYSMIS.
      COMPUTE #P=!P.
      COMPUTE #N=!N.
      
*     Test parameters P,N.
      DO IF (SYSMIS(#P) OR SYSMIS(#N)).
         COMPUTE #ERROR=1.
      ELSE.
         IF (#P<=0 OR #P>=1 OR #N<=0 OR MOD(#N,1)<>0) #ERROR=1.
      END IF.

*     Detection of the keyword THRU: Krange parameter.
      !IF (!K=!NULL) !THEN.
         !LET !Krange=!CONCAT('0 TO ',!N).
      !ELSE.
         !LET !Krange=!UPCASE(!K).
         !IF (!INDEX(!Krange,'THRU')<>0) !THEN.
            !LET !Krange=!CONCAT(!HEAD(!K),' TO',!TAIL(!TAIL(!K))).
         !IFEND.
      !IFEND.

*     Test K list.
      DO REPEAT K=!Krange.
         DO IF (MOD(K,1)<>0).
            COMPUTE #ERROR=1.
         ELSE IF (K<0).
            COMPUTE #ERROR=1.
         END IF.      
      END REPEAT.
      
      DO IF (#ERROR=1).
         PRINT /'ERROR: Parameter(s) with wrong values'.
      ELSE .
*        Correct parameters: compute results and print.
         !IF (!NST !NE !NULL) !THEN .	/*Print NST if available.
            STRING #NST (A131).
            COMPUTE #NST = CONCAT('STUDY: ',!NST).
            PRINT /#NST.
         !IFEND.
         PRINT /'BINOMIAL DISTRIBUTION: B(' !QUOTE(!N) '; ' !QUOTE(!P) ')'.

         COMPUTE #TOTAL=0.
         COMPUTE #MEAN=#N*#P.
         COMPUTE #VARIANC=#N*#P*(1-#P).

         PRINT//'+-----------+-------------+-------------+-------------+'.
         PRINT /'|     K     |   Pr(S = K) |  Pr(S <= K) |  Pr(S >= K) |'.
         PRINT /'+-----------+-------------+-------------+-------------+'.
         DO REPEAT K=!Krange.
            COMPUTE #K=K.
            COMPUTE #BINOM= CDF.BINOM(K,#N,#P)-CDF.BINOM(K-1,#N,#P).
            COMPUTE #TOTAL= #BINOM+#TOTAL.
            COMPUTE #cBINOM =CDF.BINOM(K,#N,#P).       /*Cumulative probability.
            COMPUTE #c1BINOM= 1-CDF.BINOM(K-1,#N,#P).  /*1 - Cumulative probability.
            PRINT /'|' #K(T4,F4) '|'13 #BINOM(T17,F9.8) '|'27 #cBINOM(T31,F9.8) '|'41 #c1BINOM(T45,F9.8) '|'55.
         END REPEAT.

         PRINT /'+-----------+-------------+-------------+-------------+'.
         PRINT /'|   Total   |' #total(T17,F9.8) '|'27  '|'41         '|'55.
         PRINT /'+-----------+-------------+-------------+-------------+'.
         PRINT /'     Mean =' #MEAN(F12.5).
         PRINT /' Variance =' #VARIANC(F12.5).
      END IF.
   !IFEND.
!IFEND.
EXECUTE.
RESTORE.
!ENDDEFINE.
RESTORE.