PRESERVE.
SET PRINTBACK=NONE ERRORS=NONE.
DO IF ($CASENUM=1).
  PRINT /'Macro !BONFE V2011.07.29 loaded.'
        /'Execute !BONFE HELP. for printing Help & Macro bibliographic reference.'/.
END IF.
EXECUTE.
DEFINE !BONFE (VarP =!CHAREND('/') 
		          /ID =!CHAREND('/') 
		           /P =!CHAREND('/') 
	         /METHOD =!CHAREND('/') !DEFAULT ('All')
		        /SORT =!CHAREND('/') !DEFAULT ('Or') 
               /DIR =!CHAREND('/')
		         /NST =!CHAREND('/')
              /HELP =!CHAREND('/') !DEFAULT(1) ).
PRESERVE.
SET PRINTBACK=NONE LENGHT=NONE WIDTH=132 ERRORS=NONE.
!LET !VERSION=!QUOTE('2011.07.27').
!LET !YEAR=!QUOTE(!SUBSTR(!VERSION,2,4)).
DO IF ($CASENUM=1).
   PRINT /'Macro !BONFE V' !VERSION ' (c)JM.Domenech'.
END IF.
!IF (!HELP<>1) !THEN.
*  Print HELP.
   DO IF ($casenum=1).
      PRINT /'********************************************************************************'
            /'* BONFERRONI PROCEDURES for Multiple Significance Tests                        *'
            /'* Created 2007.02.19 Last revised ' !VERSION                                  '*'80
            /'* (c) JM.Domenech                                                              *'
            /'* Programmer: R.Sesma                                                          *'
            /'*                                                                              *'
            /'* Vancouver reference:                                                         *'
            /'* Domenech JM. Macro !BONFE for SPSS Statistics.                               *'
            /'*    Bonferroni Procedures for Multiple Significance Tests [computer program]. *'
            /'*    V' !VERSION '. Bellaterra: Universitat Autnoma de Barcelona; ' !YEAR '.' '*'80
            /'*    Available from: http://www.metodo.uab.cat/macros.htm.                     *'
            /'*                                                                              *'
            /'* Macro Call (tested for SPSS 15.0 and higher):                                *'
            /'* Parameter for PRINTING MACRO DOCUMENTATION ----------------------------------*'
            /'* !BONFE HELP.                                                                 *'
            /'*                                                                              *'
            /'* Parameters for INDIVIDAL DATA -----------------------------------------------*'
            /'* !BONFE VarP = Variable name of non-corrected p-values                        *'
            /'*        [/ID]= Variable name with identifiers of non-corrected p-values       *'
            /'*               Maximum 8 characters                                           *'
            /'*               By default: sequential order in data file                      *'
            /'*                                                                              *'
            /'* Parameters for AGGREGATED DATA ----------------------------------------------*'
            /'* !BONFE    P = List of non-corrected p-Value     (1 <= P <= 0)                *'
            /'*                                                                              *'
            /'* Parameters for BOTH TYPES OF DATA -------------------------------------------*'
            /'*    [/METHOD]= Correcting methods: Bon=Bonferroni; Hol=Holm; Fin=Finner       *'
            /'*                                   Sim=Simes; All=All methods(Default)        *'
            /'*      [/SORT]= Output listing order                                           *'
            /'*               Ap: Ascending p-values; Dp: Descending p-values                *'
            /'*               Or: Original p-values order(Default)                           *'
            /'*       [/NST]= Name of the study (Label)                                      *'
            /'*                                                                              *'
            /'* Note: Parameters enclosed in square brackets [ ] are optional                *'
            /'*                                                                              *'
            /'* WARNING: If you are working with INDIVIDUAL DATA, the macro provides wrong   *'
            /'*          results if your data are filtered. To work with a set of cases      *'
            /'*          you must use SELECT IF procedure in order to create a new data      *'
            /'*          file containing only selected cases.                                *'
            /'*                                                                              *'
            /'* Samples of calls:                                                            *'
            /'*  !BONFE HELP.                                                                *'
            /'*  !BONFE VarP=Pval.                                                           *'
            /'*  !BONFE VarP=Pval /ID=Comp /METHOD=Fin /NST="Name of the Study".             *'
            /'*  !BONFE P= 0.719 0.05 0.0002 0.030 0.063 1.0 0.125 .146 1.0 0.500.           *'
            /'********************************************************************************'.
   END IF.
   EXECUTE.
!ELSE.
*  Test Parameters.
   !LET !ERROR = 0.
   !LET !METHOD0 = !UPCASE(!METHOD).
   !LET !SORT0 = !UPCASE(!SORT).
   !IF (!VarP=!NULL !AND !P=!NULL) !THEN.
      !LET !ERROR = 1.
   !ELSE.
      !IF (!P<>!NULL !AND (!ID<>!NULL !OR !VarP<>!NULL)) !THEN.
         !LET !ERROR = 2.
      !ELSE.
         !IF (!METHOD0<>'BON' !AND !METHOD0<>'HOL' !AND !METHOD0<>'FIN' !AND !METHOD0<>'SIM' !AND !METHOD0<>'ALL') !THEN.
            !LET !ERROR = 3.
         !IFEND.
         !IF (!SORT0<>'AP' !AND !SORT0<>'DP' !AND !SORT0<>'OR') !THEN.
            !LET !ERROR = 4.
         !IFEND.
      !IFEND.
   !IFEND.
*  Print error messages.
   TEMPORARY.
   N 1.
   !IF (!ERROR=1) !THEN.
      PRINT /'ERROR: Missing parameter VarP or P. See the HELP.'.
   !IFEND.
   !IF (!ERROR=2) !THEN.
      PRINT /'ERROR: Too much parameters.'.
      PRINT /'       Parameters ID and VarP are incompatible with P. See the HELP.'.
   !IFEND.
   !IF (!ERROR=3) !THEN.
      PRINT /'ERROR: Parameter METHOD with wrong values. See the HELP.'.
   !IFEND.
   !IF (!ERROR=4) !THEN.
      PRINT /'ERROR: Parameter SORT with wrong values. See the HELP.'.
   !IFEND.
   EXECUTE.

   !IF (!ERROR = 0) !THEN.
*     Bonferroni procedures.
*     Statistics in Medicine,16(22):2511-28(1997).
      !IF (!P<>!NULL) !THEN.
         DATASET NAME @7842original.      /*Name the original dataset.

*        Count the number of elements in P.
         !LET !COUNT=''.
         !DO !I !IN (!P).
            !LET !COUNT=!CONCAT(!COUNT,' ').
         !DOEND.
         !LET !COUNT=!LENGTH(!COUNT).   

*        Create a new dataset with the P aggregated data.
         INPUT PROGRAM.
         LOOP #ID=1 to !COUNT.
            COMPUTE @7842P=1.
            END CASE.
         END LOOP.
         END FILE.
         END INPUT PROGRAM.
         DATASET NAME @7842temp WINDOW=FRONT.
         EXECUTE.
         
         COMPUTE @COUNT=1.
         DO REPEAT P=!P. 
            DO IF ($CASENUM=@COUNT).
              COMPUTE @7842P=P.
            END IF.
            COMPUTE @COUNT=@COUNT+1.
         END REPEAT.

         STRING @7842ID (A8).
         COMPUTE @7842ID = STRING($CASENUM,F8).
      !ELSE.
         COMPUTE @7842P=!VarP.
         STRING @7842ID (A8).
         !IF (!ID = !NULL) !THEN.
            COMPUTE @7842ID = STRING($CASENUM,F8).
         !ELSE.
            COMPUTE @7842ID = !ID.
         !IFEND.
      !IFEND.
      RANK @7842P /N INTO @7842N /PRINT=NO.
      SORT CASES BY @7842P(A).
      COMPUTE @7842J = $CASENUM.

*     Bonferroni correction.
      COMPUTE @7842Bonfe = MIN(@7842P*@7842N,1).

*     Holm correction.
      COMPUTE @7842Holm = MIN(@7842P*(@7842N-@7842J+1),1).
      IF (@7842Holm < LAG(@7842Holm,1)) @7842Holm = LAG(@7842Holm,1).

*     Finner correction.
      COMPUTE @7842Finner = 1-(1-@7842P)**(@7842N/@7842J).
      IF (@7842Finner < LAG(@7842Finner,1)) @7842Finner = LAG(@7842Finner,1).

*     Simes correction.
      SORT CASES BY @7842P(D).
      COMPUTE @7842Simes = MIN((@7842P/@7842J)*@7842N,1).
      IF (@7842Simes > LAG(@7842Simes,1)) @7842Simes = LAG(@7842Simes,1).

*     TEST NUMERIC PARAMETER P.
      COMPUTE @7842ERROR=0.
      !IF (!P<>!NULL) !THEN.
         DO REPEAT P=!P.
            COMPUTE #temp = P.
            DO IF (SYSMIS(#temp)).
               COMPUTE @7842ERROR=1.
            ELSE.
               IF (#temp<0 OR #temp>1) @7842ERROR=1.
            END IF.
         END REPEAT.
      !IFEND.

*     Sort results by SORT.
      !IF (!SORT0='OR') !THEN.
         SORT CASES BY @7842ID(A).
      !ELSE.
         !IF (!SORT0='AP') !THEN.
            SORT CASES BY @7842P(A).
         !ELSE.
            SORT CASES BY @7842P(D).
         !IFEND.
      !IFEND.

*     Print table results, with all possibles variants.
      !IF (!METHOD0='ALL') !THEN.
         !LET !TITLE1 = 'Identifier|Non-corrected|------- CORRECTED p-values -------|'.
         !LET !TITLE2 = '          |   p-values  |Bonferroni|  Holm | Finner| Simes |'.
         !LET !TITLE3 = '----------+-------------+----------+-------+-------+-------+'.
      !ELSE.
         !LET !TITLE1 = 'Identifier|Non-corrected| CORRECTED |'.
         !IF (!METHOD0='BON') !THEN.
            !LET !TITLE2 = '          |   p-values  | Bonferroni|'.
         !IFEND.
         !IF (!METHOD0='HOL') !THEN.
            !LET !TITLE2 = '          |   p-values  |    Holm   |'.
         !IFEND.
         !IF (!METHOD0='FIN') !THEN.
            !LET !TITLE2 = '          |   p-values  |   Finner  |'.
         !IFEND.
         !IF (!METHOD0='SIM') !THEN.
            !LET !TITLE2 = '          |   p-values  |   Simes   |'.
         !IFEND.
         !LET !TITLE3 = '----------+-------------+-----------+'.
      !IFEND.

      TEMPORARY.
      N 1.
      DO IF (@7842ERROR=1).
         PRINT /'ERROR: Parameter P with wrong values. See the HELP.'.
      ELSE.
*        Print header.
         PRINT/ 'BONFERRONI PROCEDURES for Multiple Significance Tests'.
*        Print NST if available.
         !IF (!NST !NE !NULL) !THEN .
            STRING #NST (A131).
            COMPUTE #NST = CONCAT('STUDY: ',!NST).
            PRINT /#NST.
         !IFEND.

         PRINT //!QUOTE(!TITLE1).
         PRINT  /!QUOTE(!TITLE2).
         PRINT  /!QUOTE(!TITLE3).
      END IF.
      EXECUTE.

      DO IF (@7842ERROR=0).
         !IF (!METHOD0='ALL') !THEN.
            PRINT /@7842ID(A8) '|' 11 @7842P(T15,F8.6) '  |' @7842Bonfe(T29,F5.3) '  |' @7842Holm(T38,F5.3) ' |' @7842Finner(T46,F5.3) ' |' @7842Simes(T54,F5.3) ' |'.
         !IFEND.
         !IF (!METHOD0='BON') !THEN.
            PRINT /@7842ID(A8) '|' 11 @7842P(T15,F8.6) '  |' @7842Bonfe(T29,F5.3) '|' 37.
         !IFEND.
         !IF (!METHOD0='HOL') !THEN.
            PRINT /@7842ID(A8) '|' 11 @7842P(T15,F8.6) '  |' @7842Holm(T29,F5.3) '|' 37.
         !IFEND.
         !IF (!METHOD0='FIN') !THEN.
            PRINT /@7842ID(A8) '|' 11 @7842P(T15,F8.6) '  |' @7842Finner(T29,F5.3) '|' 37.
         !IFEND.
         !IF (!METHOD0='SIM') !THEN.
            PRINT /@7842ID(A8) '|' 11 @7842P(T15,F8.6) '  |' @7842Simes(T29,F5.3) '|' 37.
         !IFEND.
      END IF.
      EXECUTE.
      
      TEMPORARY.
      N 1.
      DO IF (@7842ERROR=0).
         PRINT / !QUOTE(!TITLE3).
      END IF.
      EXECUTE.

*     Restore the original data.
      !IF (!P<>!NULL) !THEN.
         DATASET ACTIVATE @7842original WINDOW=FRONT.
         DATASET CLOSE @7842temp.
      !ELSE.
         SORT CASES BY @7842ID(A).
         DELETE VARIABLES @7842ERROR @7842P @7842ID @7842N @7842J @7842Bonfe @7842Holm @7842Finner @7842Simes.
      !IFEND.
   !IFEND.
!IFEND.
EXECUTE.
RESTORE.
!ENDDEFINE.
RESTORE.