PRESERVE.
SET PRINTBACK=NONE ERRORS=NONE.
DO IF ($casenum=1).
  PRINT /'Macro !POISSON V2010.11.22 loaded.'
        /'Execute !POISSON HELP. for printing Help & Macro bibliographic reference.'/.
END IF.
EXECUTE.
DEFINE !POISSON ( K =!CHAREND('/') 
                 /L =!CHAREND('/')
               /NST =!CHAREND('/')   
              /HELP =!CHAREND('/')!DEFAULT(1) ).
PRESERVE.
SET PRINTBACK=NONE ERRORS=NONE LENGHT=NONE WIDTH=132 .
!LET !VERSION=!QUOTE('2010.11.22').
!LET !YEAR=!QUOTE(!SUBSTR(!VERSION,2,4)).
DO IF ($casenum=1).
   PRINT //'Macro !POISSON V' !VERSION ' (c)JM.Domenech & R.Granero'.
END IF.
TEMPORARY.
N 1.
!IF (!HELP<>1) !THEN.
*  Print HELP.
   DO IF ($casenum=1).
      PRINT /'******************************************************************************'
            /'* POISSON DISTRIBUTION                                                       *'
            /'* Created 1999.12.13 Last revised ' !VERSION                                '*'78
            /'* (c) JM.Domenech & R.Granero                                                *'
            /'* Programmers: A.Bonillo & R.Sesma                                           *'
            /'*                                                                            *'
            /'* Vancouver reference:                                                       *'
            /'* Domenech JM, Granero R. Macro !POISSON for SPSS Statistics.                *'
            /'*    Poisson Distribution [computer program].                                *'
            /'*    V' !VERSION '. Bellaterra: Universitat Autnoma de Barcelona; ' !YEAR '.' '*'78
            /'*    Available from: http://www.metodo.uab.cat/macros.htm.                   *'
            /'*                                                                            *'
            /'* Macro Call (tested for SPSS 15.0 and higher):                              *'
            /'* !POISSON  K = List with the number of events in the sample (Integer >= 0)  *'
            /'*          /L = Lambda parameter  or  p*n                    (L > 0)         *'
            /'*       [/NST]= Name of the study (Label)                                    *'
            /'*       [/HELP] Print Macro call arguments and Samples of calls              *'
            /'*                                                                            *'
            /'* Note: Parameters enclosed in square brackets [ ] are optional              *'
            /'*                                                                            *'
            /'* Samples of calls:                                                          *'
            /'*  !POISSON HELP.                                                            *'
            /'*  !POISSON K=1 2 6 /L=2 /NST="Name of the Study".                           *'
            /'*  !POISSON K=1 THRU 6 /L=0.02*100 .                                         *'
            /'******************************************************************************'.
   END IF.
   EXECUTE.
!ELSE.
*  Test the parameters.
   !IF (!K=!NULL !or !L=!NULL) !THEN 
      PRINT /'ERROR: Missing parameter'.
   !ELSE.
      COMPUTE #ERROR=0.
      
*     Test parameter L.
      COMPUTE #L=$SYSMIS.
      COMPUTE #L=!L.
      DO IF (SYSMIS(#L)).
         COMPUTE #ERROR=1.
      ELSE.
         IF (#L<=0) #ERROR=1.
      END IF.            
      
*     Detection of the keyword THRU: Krange parameter.
      !LET !Krange=!UPCASE(!K).
      !IF (!INDEX(!Krange,'THRU')<>0) !THEN.
         !LET !Krange=!CONCAT(!HEAD(!K),' TO',!TAIL(!TAIL(!K))).
      !IFEND.
      
*     Test K list.
      DO REPEAT K=!Krange.
         DO IF (MOD(K,1)<>0).
            COMPUTE #ERROR=1.
         ELSE IF (K<0).
            COMPUTE #ERROR=1.
         END IF.      
      END REPEAT.

      DO IF (#ERROR=1).
         PRINT /'ERROR: Parameter(s) with wrong values'.
      ELSE.
*        Correct parameters: compute results and print.
         !IF (!NST !NE !NULL) !THEN .	/*Print NST if available.
            STRING #NST (A131).
            COMPUTE #NST = CONCAT('STUDY: ',!NST).
            PRINT /#NST.
         !IFEND.
         PRINT /'POISSON DISTRIBUTION: P('!QUOTE(!L)')'.

        COMPUTE #MEAN=#L.
        COMPUTE #VARIANC=#L.
        PRINT /'                Mean='#MEAN(F12.5).
        PRINT /'            Variance='#VARIANC(F12.5).
         PRINT//'+-----------+-------------+-------------+-------------+'.
         PRINT /'|     K     |   Pr(S = K) |  Pr(S <= K) |  Pr(S >= K) |'.
         PRINT /'+-----------+-------------+-------------+-------------+'.

         DO REPEAT K=!Krange.
            COMPUTE #K=K.
            COMPUTE #POISSON= CDF.POISSON(K,#L)-CDF.POISSON(K-1,#L).
            COMPUTE #TOTAL= #POISSON+#TOTAL.
            COMPUTE #cPOISSON= CDF.POISSON(K,#L).       /*Cumulative probability.
            COMPUTE #c1POISSON= 1-CDF.POISSON(K-1,#L).  /*1 - Cumulative probability.
            PRINT /'|' #K(T4,F4) '|'13 #POISSON(T17,F9.8) '|'27 #cPOISSON(T31,F9.8) '|'41 #c1POISSON(T45,F9.8) '|'55.
         END REPEAT.
         PRINT /'+-----------+-------------+-------------+-------------+'.
         PRINT /'|   Total   |' #total(T17,F9.8) '|'27  '|'41         '|'55.
         PRINT /'+-----------+-------------+-------------+-------------+'.
      END IF.
   !IFEND.
!IFEND.
EXECUTE.
RESTORE.
!ENDDEFINE.
RESTORE.
