PRESERVE.
SET PRINTBACK=NONE ERRORS=NONE.
DO IF ($casenum=1).
  PRINT /'Macro !CII V2008.02.29 loaded.'
        /'Execute !CII HELP. for printing Help & Macro bibliographic reference.'/.
END IF.
EXECUTE.
DEFINE !CII (A =!CHAREND('/')
            /T =!CHAREND('/')
           /CL =!CHAREND('/')!DEFAULT('95')
 	      /PrCI =!CHAREND('/')!DEFAULT('E')
 	       /NST =!CHAREND('/')
         /HELP =!CHAREND('/')!DEFAULT(1) ).

PRESERVE.
SET PRINTBACK=NONE ERRORS=NONE LENGHT=NONE WIDTH=132 .
!LET !VERSION=!QUOTE('2008.02.29').
!LET !YEAR=!QUOTE(!SUBSTR(!VERSION,2,4)).
DO IF ($casenum=1).
   PRINT //'Macro !CII V' !VERSION ' (c)JM.Domenech & R.Granero'.
END IF.
TEMPORARY.
N 1.
!IF (!HELP<>1) !THEN.
*  Print HELP.
   DO IF ($casenum=1).
      PRINT /'******************************************************************************'
            /'* CONFIDENCE INTERVALS FOR INCIDENCE RATES                                   *'
            /'* Created 2001.12.30 Last revised ' !VERSION                                '*'78
            /'* (c) JM.Domenech & R.Granero                                                *'
            /'* Programmers: A.Bonillo & R.Sesma                                           *'
            /'*                                                                            *'
            /'* Vancouver reference:                                                       *'
            /'* Domenech JM, Granero R. Macro !CII for SPSS Statistics.                    *'
            /'*    Confidence Intervals for Incidence Rates [computer program].            *'
            /'*    V' !VERSION '. Bellaterra: Universitat Autnoma de Barcelona; ' !YEAR '.' '*'78
            /'*    Available from: http://www.metodo.uab.cat/macros.htm.                   *'
            /'*                                                                            *'
            /'* Macro Call (tested for SPSS 15.0 and higher):                              *'
            /'* !CII A = List of Events                      (Ai: integer >= 0)            *'
            /'*     /T = List of person-time exposed         (Ti >= Ai and > 0)            *'
            /'*   [/CL]= Confidence level(%); by default: 95 (50 <= CL < 100)              *'
            /'*        = SE  for printing Standard Error instead of Confidence Interval    *'
            /'* [/PrCI]= Method for compute CI of proportions:                             *'
            /'*          E  Exact (Poisson)                   by default: E                *'
            /'*          W  Wilson                                                         *'
            /'*          A  Asymptotic                                                     *'
            /'*          ALL                                                               *'
            /'*  [/NST]= Name of the study (Label)                                         *'
            /'*  [/HELP] Print Macro call arguments and Samples of calls                   *'
            /'*                                                                            *'
            /'* Note: Parameters enclosed in square brackets [ ] are optional              *'
            /'*                                                                            *'
            /'*                               +------------+----------+----------+-----+   *'
            /'* Samples of calls:             |            | Sample 1 | Sample 2 | ... |   *'
            /'* !CII HELP.                    +------------+----------+----------+-----+   *'
            /'* !CII A=9 /T=30000.            |     Events |    A1    |    A2    | ... |   *'
            /'* !CII A=9 /T=30000 /PrCI=ALL.  +------------+----------+----------+-----+   *'
            /'* !CII A=9 0 /T=300 10          | Person-Time|    N1    |    N2    | ... |   *'
            /'*     /NST="Study Name" /CL=95. +------------+----------+----------+-----+   *'
            /'******************************************************************************'.
   END IF.
   EXECUTE.
!ELSE. 
*  Test the parameters.
   !IF (!A=!NULL !or !T=!NULL) !THEN 
      PRINT /'ERROR: Missing parameter'.
   !ELSE.
      COMPUTE #ERROR=0.
      
*     Count of the numbers of elements.
      COMPUTE #COUNTA=0.
      DO REPEAT A=!A.
         COMPUTE #COUNTA=#COUNTA+1.
      END REPEAT.
      COMPUTE #COUNTT=0.
      DO REPEAT T=!T.
         COMPUTE #COUNTT=#COUNTT+1.
      END REPEAT.
      IF (#COUNTA<>#COUNTT) #ERROR=1.
      
*     Test parameters A,T.
      DO REPEAT A=!A /T=!T.
         COMPUTE #A=$SYSMIS.
         COMPUTE #T=$SYSMIS.
         COMPUTE #A=A.
         COMPUTE #T=T.
         DO IF (SYSMIS(#A) OR SYSMIS(#T)).
            COMPUTE #ERROR=1.
         ELSE.
            IF (MOD(#A,1)<>0 ) #ERROR=2.
            IF (#A<0 OR #T<=0 OR #A>#T) #ERROR=1.
         END IF.
      END REPEAT.
   
*     Test parameter PrCI.
      COMPUTE #PrCI=0.       /*Method for compute CI: W (Wilson) = 1; E (Exact) = 2-default; A (Asymptotic) = 3; All = 4.
      !LET !PrCIU=!UPCASE(!PrCI).
      !IF (!PrCIU<>'W' !AND !PrCIU<>'E' !AND !PrCIU<>'A' !AND !PrCIU<>'ALL') !THEN.
         COMPUTE #ERROR=1.
      !ELSE.
         IF (!QUOTE(!PrCIU)='W') #PrCI=1.
         IF (!QUOTE(!PrCIU)='E') #PrCI=2.
         IF (!QUOTE(!PrCIU)='A') #PrCI=3.
         IF (!QUOTE(!PrCIU)='ALL') #PrCI=4.
      !IFEND.

*     Test parameter CL.
      COMPUTE #SE=0. 
      COMPUTE #CL=$SYSMIS.
      !IF (!UPCASE(!CL)='SE') !THEN.
         COMPUTE #SE=1.
         !LET !TITLE = "Asymptotic Standard Error".
         COMPUTE #CL = 95.
      !ELSE.
         COMPUTE #CL = !CL.
         !LET !TITLE = !CONCAT(!CL,'% Confidence Interval ').
         DO IF (SYSMIS(#CL)).
            COMPUTE #ERROR=1.
         ELSE.
            IF (!CL<50 or !CL>=100) #ERROR=1.
         END IF.
      !IFEND.

      DO IF (#ERROR=1).
         PRINT /'ERROR: Parameter(s) with wrong values'.
      ELSE IF (#ERROR=2).
         PRINT /'ERROR: Parameter(s) with decimal values'.
      ELSE .
*        Correct parameters: compute of variables and print.
         COMPUTE #WARNING=0.
         COMPUTE #RECOM=0.
         COMPUTE #P = (#CL+100)/200.
         COMPUTE #Z = IDF.NORMAL(#P,0,1).
   
         PRINT  /'CONFIDENCE INTERVALS FOR INCIDENCE RATES'.
         !IF (!NST !NE !NULL) !THEN .	/*Print NST if available.
            STRING #NST (A131).
            COMPUTE #NST = CONCAT('STUDY: ',!NST).
            PRINT /#NST.
         !IFEND.
         
         PRINT //'Events  Person_time | Estimate | ' !QUOTE(!TITLE).
         PRINT  /'--------------------+----------+---------------------------------'.
   
         STRING #Tn(A1).
         COMPUTE #COUNT = 0.
         DO REPEAT A=!A /T=!T. 
            COMPUTE #COUNT = #COUNT+1.
            COMPUTE #A = A.
            COMPUTE #T = T.
            COMPUTE #M = #A / #T.
            
            COMPUTE #lbP= 0.             /*Poisson bounds.
            COMPUTE #ubP= #T.
            IF (#A > 0) #lbP= 0.5*IDF.CHISQ(1-#P, 2*#A).
            IF (#A < #T)#ubP= 0.5*IDF.CHISQ(#P, 2*#A+2).
            COMPUTE #lbE= #lbP/#T.      /*Exact bounds.
            COMPUTE #ubE= #ubP/#T.
            
            COMPUTE #WA = 2*#A + #Z**2.   /*Wilson bounds.
            COMPUTE #WB = #Z * SQRT(#Z**2 + 4*#A*(#T-#A)/#T).
            COMPUTE #WC = 2*(#T + #Z**2).
            COMPUTE #lbW= (#WA-#WB)/#WC.
            COMPUTE #ubW= (#WA+#WB)/#WC.
            
            COMPUTE #SEA= SQRT(#A)/#T.    /*Asymptotic bounds.
            COMPUTE #lbA= #M - #SEA*#Z.
            COMPUTE #ubA= #M + #SEA*#Z.

            DO IF (#SE=0).
               PRINT /#A(F6) '|'21 #A(T23,F8) '|'32 #lbP(F9.4) ' to' #ubP(F9.4) '  Poisson'.
               PRINT /'|'21 '|'32.
               COMPUTE #Tn=' '.
               IF (#ubA>1 OR #lbA<0) #Tn='*'.
               IF ((#ubA>1 OR #lbA<0) AND (#PrCI>2)) #WARNING=1.
               DO IF (#PrCI=1).
                  PRINT /#A(F6)  #T(T9,F11.1) '|'21 #M(T23,F8.7) '|'32 #lbW(T34,F8.7) ' to ' #ubW(T46,F8.7) '  Wilson'.
               ELSE IF (#PrCI=2).
                  PRINT /#A(F6)  #T(T9,F11.1) '|'21 #M(T23,F8.7) '|'32 #lbE(T34,F8.7) ' to ' #ubE(T46,F8.7) '  Exact'.
               ELSE IF (#PrCI=3).
                  PRINT /#A(F6)  #T(T9,F11.1) '|'21 #M(T23,F8.7) '|'32 #lbA(T34,F8.7) ' to ' #ubA(T46,F8.7) #Tn(A1) ' Asymptotic'.
               ELSE IF (#PrCI=4).
                  PRINT /#A(F6)  #T(T9,F11.1) '|'21 #M(T23,F8.7) '|'32 #lbE(T34,F8.7) ' to ' #ubE(T46,F8.7) '  Exact'.
                  PRINT /                     '|'21              '|'32 #lbW(T34,F8.7) ' to ' #ubW(T46,F8.7) '  Wilson'.
                  PRINT /                     '|'21              '|'32 #lbA(T34,F8.7) ' to ' #ubA(T46,F8.7) #Tn(A1) ' Asymptotic'.
                  COMPUTE #RECOM=1.
               END IF.
            ELSE.
               PRINT /#A(F6)  #T(T9,F11.1) '|'21 #M(T23,F8.7) '|'32 #SEA(T34,F10.8).
            END IF.
            
*           Print the separate line.
            DO IF (#COUNT <> #COUNTA).
               PRINT /'|'21 '|'32.
            END IF.
         END REPEAT.
         PRINT  /'--------------------+----------+---------------------------------'.
         DO IF (#ERROR=0).
            DO IF (#WARNING=1).
               PRINT /'(*) Out of Range'.
            END IF.
            DO IF (#RECOM=1).
               PRINT /'Recommended CI assuming Poisson distribution'.
            END IF.
         END IF.
      END IF.
   !IFEND.
!IFEND.
EXECUTE.
RESTORE.
!ENDDEFINE.
RESTORE.

