PRESERVE.
SET PRINTBACK=NONE ERRORS=NONE.
DO IF ($casenum=1).
   PRINT /'Macro !RNDP V2011.09.09 loaded.'
         /'Execute !RNDP HELP. for printing Help & Macro bibliographic reference.'/.
END IF.
EXECUTE.
DEFINE !RNDP (LK=!CHAREND('/') /HK=!CHAREND('/')
              /N=!CHAREND('/') 
           /SEED=!CHAREND('/') !DEFAULT('RANDOM')
            /NST=!CHAREND('/')  
           /HELP=!CHAREND('/') !DEFAULT(1)).
PRESERVE.
SET PRINTBACK=NONE ERRORS=NONE LENGHT=NONE WIDTH=132 .
!LET !VERSION=!QUOTE('2011.09.09').
!LET !YEAR=!QUOTE(!SUBSTR(!VERSION,2,4)).
DO IF ($casenum=1).
   PRINT //'Macro !RNDP V' !VERSION ' (c)JM.Domenech & R.Granero'.
END IF.
!IF (!HELP<>1) !THEN.
*  Print HELP.
   DO IF ($casenum=1).
      PRINT /'******************************************************************************'
            /'* CREATE A N RANDOM PERMUTATIONS OF THE K ELEMENTS                           *'
            /'* Created 1999.03.09 Last revised ' !VERSION                                '*'78
            /'* (c) JM.Domenech & R.Granero                                                *'
            /'* Programmers: A.Bonillo & R.Sesma                                           *'
            /'*                                                                            *'
            /'* Vancouver reference:                                                       *'
            /'* Domenech JM, Granero R. Macro !RNDP for SPSS Statistics.                   *'
            /'*    Create a N Random Permutations of the K Elements [computer program].    *'
            /'*    V' !VERSION '. Bellaterra: Universitat Autnoma de Barcelona; ' !YEAR '.' '*'78
            /'*    Available from: http://www.metodo.uab.cat/macros.htm.                   *'
            /'*                                                                            *'
            /'* Macro Call (tested for SPSS 15.0 and higher):                              *'
            /'* !RNDP LK = Low element: Integer (>=0) or Letter (English alphabet)         *'
            /'*      /HK = High element: Integer or Letter           (HK > LK)             *'
            /'*       /N = Number of random permutations             (Integer > 0)         *'
            /'*   [/SEED]= Initialization value; by default: RANDOM  (any real number)     *'
            /'*            The Mersenne Twister (MT) random number generator is used       *'
            /'*    [/NST]= Name of the study (Label)                                       *'
            /'*    [/HELP] Print Macro call arguments and Samples of calls                 *'
            /'*                                                                            *'
            /'* Note: Parameters enclosed in Square Brackets [ ] are optional              *'
            /'*                                                                            *'
            /'* Samples of calls:                                                          *'
            /'* !RNDP HELP.                                                                *'
            /'* !RNDP LK=0 /HK=9 /N=25 /SEED= 1769583145.                                  *'
            /'* !RNDP LK=A /HK=F /N=40 /NST="Name of the Study".                           *'
            /'******************************************************************************'.
   END IF.
   EXECUTE.
!ELSE.
*  Test parameters.
   !LET !ERROR = 0.
   !IF (!LK=!NULL !OR !HK=!NULL !OR !N=!NULL) !THEN.
      !LET !ERROR = 1.        /*Missing parameters.
   !ELSE.   
*     Are LK and HK numbers?.
      !LET !LKU=!UPCASE(!LK).
      !LET !HKU=!UPCASE(!HK).
      !LET !ALFABET=ABCDEFGHIJKLMNOPQRSTUVWXYZ.
      !LET !POS1=!INDEX(!ALFABET,!LKU) .
      !LET !POS2=!INDEX(!ALFABET,!HKU) .
      !IF (!POS1=0 !AND !POS2=0) !THEN.
         !LET !NUM = 1.
      !ELSE.
         !IF (!POS1<>0 !AND !POS2<>0) !THEN.
            !LET !NUM = 0.
         !ELSE.
            !IF (!LENGTH(!LKU)<>1) !THEN.
               !LET !ERROR = 3.     /*LK must be only one letter.
            !ELSE.
               !IF (!LENGTH(!HKU)<>1) !THEN.
                  !LET !ERROR = 4.     /*HK must be only one letter.
               !ELSE.
                  !IF ((!POS1<>0 !AND !POS2=0) !OR (!POS1=0 !AND !POS2<>0)) !THEN.
                     !LET !ERROR = 2.  /*LK and HK are not of the same type.
                  !IFEND.
               !IFEND.
            !IFEND.
         !IFEND.
      !IFEND.
      !IF (!N<=0) !THEN.
         !LET !ERROR = 5.
      !IFEND.
   !IFEND.

   !IF (!ERROR<>0) !THEN.
      TEMPORARY.
      N 1.
      !IF (!ERROR=1) !THEN.
         PRINT /'ERROR: Missing parameter LK, HK or N. See the HELP.'.
      !IFEND.
      !IF (!ERROR=2) !THEN.
         PRINT /'ERROR: LK and HK must be of the same type. See the HELP.'.
      !IFEND.
      !IF (!ERROR=3) !THEN.
         PRINT /'ERROR: Parameter LK with wrong value. See the HELP.'.
      !IFEND.
      !IF (!ERROR=4) !THEN.
         PRINT /'ERROR: Parameter HK with wrong value. See the HELP.'.
      !IFEND.
      !IF (!ERROR=5) !THEN.
         PRINT /'ERROR: Parameter N with wrong value. See the HELP.'.
      !IFEND.
      EXECUTE.
   !ELSE.
*     Name the original dataset.
      DATASET NAME @96678original.

      !IF (!NUM=1) !THEN 
        !LET !TLK=!LK.     /*LK, HK are numbers.
        !LET !THK=!HK.
      !ELSE.
        !LET !TLK=!POS1.   /*LK, HK are letters.
        !LET !THK=!POS2.
      !IFEND.

*     Substraction of parameters to find the number of elements.
      !LET !NEL=!LENGTH(!SUBSTR(!CONCAT(!BLANKS(!THK),' '),!LENGTH(!CONCAT(!BLANKS(!TLK),' ')))).

*     Allocation of seed.
      PRESERVE.
      SET RNG=MT MTINDEX= !SEED.

      INPUT PROGRAM.
*     Making the vector and assign format.
      VECTOR V(!NEL,F2).            
*     Make N registers.
      LOOP #I =1 to !N.           
*        Make NEL variables. 
         LOOP #J = 1 to !NEL.
*           Assignation of values to variables.
            COMPUTE V(#J)=!TLK+#J-1.
         END LOOP.
*        Make VV variables. 
         LOOP #J = !NEL to 2 by -1.            
*           Chose at random of a natural number between 1 and NEL-1.
            COMPUTE #IN=TRUNC(RV.UNIFORM(1,#J+1)).
*           Assignate to a scratch variable the element of the vector chose at random.
            COMPUTE #IVAL =V(#IN).
*           Assignate to random element the temporal element .
            COMPUTE V(#IN)=V(#J).
*           Assignate to element temporal of the random value.
            COMPUTE V(#J) =#IVAL.
         END LOOP.
         END CASE.
      END LOOP.
      END FILE.
      END INPUT PROGRAM.
      DATASET NAME @96678RNDP WINDOW=FRONT.
      EXECUTE.
      RESTORE.

*     Conversion of numbers to strings if its necessary.
      !LET !VHN=!CONCAT('V',!NEL).
      !LET !XHN=!CONCAT('X',!NEL).
      !IF (!NUM=0) !THEN.
         STRING X1 TO !XHN (A1).
         DO REPEAT I=V1 TO !VHN /X=X1 TO !XHN.
            COMPUTE X=SUBSTR(!QUOTE(!ALFABET),I,1).
         END REPEAT.
         MATCH FILES /FILE=* /DROP=V1 to !VHN.
         EXECUTE.
      !IFEND.

*     Test numerical parameters.
      COMPUTE #ERROR=0.
      COMPUTE #LK=$SYSMIS.
      COMPUTE #HK=$SYSMIS.
      !IF (!NUM=1) !THEN.
*        Test HK, LK.
         COMPUTE #LK = !LK.
         COMPUTE #HK = !HK.
         DO IF (SYSMIS(#LK) OR SYSMIS(#HK)).
            COMPUTE #ERROR=1.
         ELSE.
            IF (#LK<0 OR #HK<0 OR MOD(#LK,1)<>0 OR MOD(#HK,1)<>0) #ERROR=1.
         END IF.
      !ELSE.
         COMPUTE #LK = !TLK.
         COMPUTE #HK = !THK.
      !IFEND.
      IF (#HK <= #LK) #ERROR=2.
*     Test N.
      COMPUTE #N=$SYSMIS.
      COMPUTE #N = !N.
      DO IF (SYSMIS(#N)).
         COMPUTE #ERROR=3.
      ELSE.
         IF (#N<=0 OR MOD(#N,1)<>0) #ERROR=3.
      END IF.
*     Test SEED.
      !IF (!SEED<>'RANDOM') !THEN.
         COMPUTE #SEED=$SYSMIS.
         COMPUTE #SEED = !SEED.
         DO IF (SYSMIS(#SEED)).
            COMPUTE #ERROR=4.
         END IF.
      !IFEND.
      
      DO IF (#ERROR>0).
         COMPUTE @ERROR=1.
      ELSE.
         COMPUTE @ERROR=0.
      END IF.

      DO IF ($CASENUM=1).
         DO IF (#ERROR=1).
            PRINT /'ERROR: Parameter LK or HK with wrong value. See the HELP.'.
         ELSE IF (#ERROR=2).
            PRINT /'ERROR: Parameter HK must be greater than LK. See the HELP.'.
         ELSE IF (#ERROR=3).
            PRINT /'ERROR: Parameter N with wrong value. See the HELP.'.
         ELSE IF (#ERROR=4).
            PRINT /'ERROR: Parameter SEED with wrong value. See the HELP.'.
         ELSE.
            !IF (!NST !NE !NULL) !THEN .	/*Print NST if available.
               STRING #NST (A131).
               COMPUTE #NST = CONCAT('STUDY: ',!NST).
               PRINT /#NST.
            !IFEND.
   
            PRINT //'Random number seed= '!QUOTE(!SEED).
            PRINT / !QUOTE(!N) ' random permutations of ' !QUOTE(!LK) ' to ' !QUOTE(!HK) ' elements:'/.
         END IF.
      END IF.
      
      DO IF (#ERROR=0).
         !IF (!NUM=1) !THEN.
            PRINT /V1 to !VHN (!NEL(F3,2X)).
         !ELSE.
            PRINT /X1 to !XHN (!NEL(A1,2X)).
         !IFEND.
      END IF.
      EXECUTE.
      
      DELETE VARIABLES @ERROR.
   !IFEND.
!IFEND.
EXECUTE.
RESTORE.
!ENDDEFINE.
RESTORE.

